clear all
close all
clc

disp('Loss calculation for a 100W Active Clamp Forward (ACF) converter')
disp('TBD: Include switching loss in all diodes')

% Parameters
Vo    = 12
Po    = 100   % [W] Output power
Ioavg = Po/Vo % [A] Average output current 
r     = 0.18  % [-] Inductor current ripple ratio 
Dmax  = 0.71  % [-] Maximum duty ratio
Vin   = 48    % [V] Actual dc input voltage
Vin_min = 36  % [V] Minimum dc input voltage
Vin_max = 72  % [V] Maximum dc input voltage
Vin_nom = 48  % [V] Nominal dc input voltage
Vs      = 1   % [V] Sum total of series voltage drops (refered to secondary side)
Nmin  = 12/(Vin_min*Dmax)   % [-] Min. required turns ratio, sec/pri 
N     = 0.55                % [-] Actual turns ratio 
D     = (12+Vs)/(Vin*N)     % [-] Nominal duty ratio
Vds   = Vin/(1-D)           % [V] MOSFET drain-source voltage
piv2  = (Vds-Vin)*N         % [V] Main recitifer diode peak-inverse-voltage
piv3  = N*Vin               % [V] Freewheeling diode peak-inverse-voltage
fsw   = 200e3               % [Hz] Switching frequency
Tsw   = 1/fsw;              % [s] Switching cycle time
Tj    = 25;                 % [degree C] Junction temperature 
Rdson1_25= 0.030;           % [ohm] Main MOSFET ON-state resistance (base value at 25C) 
Rdson1_Tj= (1.8*Rdson1_25/150)*(Tj-25)+Rdson1_25
                            % From 1st plot on pg-4 of SQM10250E datasheet:
                            % 2.1*0.03=0.063 @125C 
Rdsonmax2= 0.064            % [ohm] Aux MOSFET ON-state resistance (worst case) 
                            % From Fig.9 on pg-6 of IPD600N25N3 datasheet:
                            % 0.120 ohm @125C 

Vfmax = 0.5                 % [V] Rectifier forward drop (worst case). E.g. 0.7, 0.6, 0.1
Csn   = 2*1e-9              % [F] Snubber capacitance for four 'atomic' diodes

diode = 2;                  % 1=STPS20L60C, 2=RBQ30NS65AFH
if diode==1 % STPS20L60C
T      = [25:25:150];                    % [deg C] Diode junction temperature full-range
Irlk20 = [0.025 0.25 1.0 5 25  90]*1e-3; % [A] Reverse leakage current of 'atomic' diode at 20V
Irlk40 = [0.050 0.40 1.5 8 35 150]*1e-3; % [A] Reverse leakage current of 'atomic' diode at 40V
Irlk20 = interp1(T,Irlk20,Tj)
Irlk40 = interp1(T,Irlk40,Tj)
m      = (Irlk40-Irlk20)/20;
Irlk2  = Irlk20+m*(piv2-20)              % [A] Leakage current in main rectifier diode at given Tj and PIV
Irlk3  = Irlk20+m*(piv3-20)              % [A] Leakage current in FWD at given Tj and PIV
end

if diode==2 % RBQ30NS65AFH
T      = [25 75 125 150];                % [deg C] Diode junction temperature full-range
Irlk20 = [0.002 0.1 2 7]*1e-3; % [A] Reverse leakage current of 'atomic' diode at 20V
Irlk40 = [0.005 0.2 3 15]*1e-3; % [A] Reverse leakage current of 'atomic' diode at 40V
Irlk20 = interp1(T,Irlk20,Tj)
Irlk40 = interp1(T,Irlk40,Tj)
m      = (Irlk40-Irlk20)/20;
Irlk2  = Irlk20+m*(piv2-20)              % [A] Leakage current in main rectifier diode at given Tj and PIV
Irlk3  = Irlk20+m*(piv3-20)              % [A] Leakage current in FWD at given Tj and PIV
end

Lm    = 76e-6;             % [H] Magnetizing inductance of transformer
Llk   = 0.85e-6;           % [H] Leakage inductance of transformer
L1    = Lm+Llk             % [H] Primary inductance of transformer
del_Im= (Vin/L1)*D*Tsw     % [A] Total change in magnetizing current
RLo   = 25e-3;             % [ohm] Resistance of output inductor
Rxfr  = 200e-3;            % [ohm] Both winding resistances reflected to primary
Resr  = 50e-3;             % [ohm] Combined ESR of all output capacitors
if diode==1 % STPS20L60C
vto   = 0.41-0.0017*(Tj-25);
rd    = 0.02+0.0002*(Tj-25);
end
if diode==2 % RBQ30NS65AFH
vto   = 0.37-0.0013*(Tj-25);
rd    = 0.025+0.00015*(Tj-25);
end
% Reference for determining vto and rd parameters: ST AN604 Application note (2011):
% 'Calculation of conduction losses in a power rectifier'

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% Secondary side calculations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Main rectifier diode 
I2max = Ioavg*(1+r/2) % [A] Maximum secondary current during t_on
I2min = Ioavg*(1-r/2) % [A] Minimum secondary current during t_on
I2a   = linspace(I2min,I2max,ceil(D*100)); % [A] Secondary current during t_on
I2b   = zeros(1,floor((1-D)*100));         % [A] Secondary current during t_off
I2    = [I2a I2b];                         % [A] Secondary current during t_on and t_off
I2avg = mean(I2)                           % [A] Average value of secondary current 
I2rms = sqrt(mean(I2.^2))                  % [A] RMS value of secondary current

I21   = I2/4;                              % [A] Main rectifier diode current per 'atomic' diode inside a D2PAK
I21avg= mean(I21)                          % [A] Average value of I21 
I21rms= sqrt(mean(I21.^2))                 % [A] RMS value of I21
Pcond21avg=vto*I21avg + rd*I21rms.^2       % Using equation in pg-2 of STPS20L60C datasheet
Pcond2d2pak=2*Pcond21avg;                  % [W] Conduction loss in single D2PAK (two diodes)
Pcond2avg = 4*Pcond21avg;                  % [W] Conduction loss in two parallel D2PAK (four diodes)

Psnub2avg = Csn*piv2^2*fsw;                % [W] Dissipation in snubber resistors of main diodes
Plk2d2pak = Irlk2*2*piv2*(1-D);            % [W] Dissipation in D2PAK (two diodes) due to reverse leakage current 
Plk2avg   = 2*Plk2d2pak;                   % [W] Average dissipation in 2 D2PAKs (4 diodes) due to reverse leakage current
P2d2pak   = Pcond2d2pak+Plk2d2pak;         % [W] Total dissipation in a D2PAK (conduction loss + leakage loss)

% Freewheeling diode (FWD)
I3a   = zeros(1,ceil(D*100));              % [A] Freewheeling diode (FWD) current during t_on
I3b   = linspace(I2max,I2min,floor((1-D)*100)); % [A] FWD current during t_off
I3    = [I3a I3b];                         % [A] FWD current during t_on and t_off
I3avg = mean(I3)                           % [A] Average value of FWD current 
I3rms = sqrt(mean(I3.^2))                  % [A] RMS value of FWD current

I31   = I3/4;                              % [A] FWD current per 'atomic' diode inside a D2PAK
I31avg= mean(I31)                          % [A] Average value of I31 
I31rms= sqrt(mean(I31.^2))                 % [A] RMS value of I31
Pcond31avg=vto*I31avg + rd*I31rms.^2       % Using equation in pg-2 of STPS20L60C datasheet
Pcond3d2pak=2*Pcond31avg;                  % [W] Conduction loss in single D2PAK (two diodes)
Pcond3avg = 4*Pcond31avg;                  % [W] Conduction loss in two parallel D2PAK (four diodes)

Psnub3avg = Csn*piv3^2*fsw;                % [W] Dissipation in snubber resistors of FWDs
Plk3d2pak = Irlk3*2*piv3*D;
Plk3avg   = 2*Plk3d2pak;
P3d2pak   = Pcond3d2pak+Plk3d2pak;

% Output Inductor (Lo)
ILo       = I2+I3;
ILorms    = sqrt(mean(ILo.^2));
PLo       = ILorms^2*RLo;

% Output Capacitor (Co)
ICo       = ILo-Ioavg;
ICorms    = sqrt(mean(ICo.^2));
PCo       = ICorms^2*Resr;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% Primary side calculations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
I1     = I2*N;              % [A] Primary current during t_on and t_off
I1max  = I2max*N
I1min  = I2min*N

Ima    = linspace(-del_Im/2,del_Im/2,ceil(D*100));
Imb    = linspace(del_Im/2,-del_Im/2,floor((1-D)*100)); 
Im     = [Ima Imb];

Ip     = I1+Im;
Ipmax  = max(Ip);
Iprms  = sqrt(mean(Ip.^2))   % [A] RMS value of primary current
I1rms  = sqrt(mean(I1.^2))   % [A] RMS value of primary current ignoring Im

% Main MOSFET (fet1)
Ifet1a = Ip(1:ceil(D*100));
Ifet1b = zeros(1,floor((1-D)*100));
Ifet1  = [Ifet1a Ifet1b]; 
Pcondfet1 = Ifet1.^2*Rdson1_Tj;   % [W] Conduction loss in main MOSFET
Pcondfet1avg = mean(Pcondfet1);   % [W] Average conduction loss in main MOSFET
Eoff1  = 0.5*20e-9*(Ipmax/2)*(Vds/2); % [J] Turn-off loss (energy) based on LTspice waveforms
Pswoff1= Eoff1*fsw                    % [W] Turn-off loss (power) 
Pfet1  = Pcondfet1avg + Pswoff1

% Aux MOSFET (fet2)
Ifet2a = zeros(1,ceil(D*100));
Ifet2b = Imb;
Ifet2  = [Ifet2a Ifet2b]; 
Pcondfet2 = Ifet2.^2*Rdsonmax2;    % [W] Conduction loss in aux MOSFET
Pcondfet2avg = mean(Pcondfet2);    % [W] Average conduction loss in aux MOSFET

% Transformer (xfr)
Pxfr_cu      = Ip.^2*Rxfr;         % [W] Copper loss profile
Pxfr_cu_avg  = mean(Pxfr_cu)       % [W] Average copper loss profile
Pxfr_fe      = 205e3*2430e-9       % [W] Iron loss for RM8 core, N87 material (Ae=55mm2)
                                   % @ fsw=200kHz, delB=delPhi/Ae=(V/N)*delT/Ae=0.2T,
                                   % Bmax=delB/2=0.1T, T=50C, Fe loss density = 205kW/m3 
                                   % core volume, v = 2430 mm3 
% Ref: https://tools.tdk-electronics.tdk.com/mdt/pl_temp
Pxfr         = Pxfr_cu_avg + Pxfr_fe

% Aux PSU (includes gate drive losses)
Vaux = 13.7;   % [V] About 3-4V higher than Vcc=10.4V
Icc  = 48e-3;  % [A] Based on measurement from v0 prototype
Pcc  = Vaux*Icc;

disp('--------------')
% Total losses
Pxfr                      % [W] Loss in transformer
Pfet1                     % [W] Loss in main MOSFET 
Pd2 = Pcond2avg + Plk2avg % [W] Loss in main rectifier diodes
Pd3 = Pcond3avg + Plk3avg % [W] Loss in frewheeling diodes
PLo                       % [W] Loss in output inductor
Psnub = Psnub2avg + Psnub3avg     % [W] Loss in snubbers
Pmisc = Pcondfet2avg + PCo + Pcc  % [w] Other (miscellaneous) losses

figure(1); p=pie([Pxfr Pfet1 Pd2 Pd3 PLo Psnub Pmisc],...
{'P_{xfr}' 'P_{fet1}' 'P_{d2}' 'P_{d3}' 'P_{Lo}' 'P_{snub}' 'P_{misc}'});

Ploss_total = Pxfr + Pfet1 + Pd2 + Pd3 + Psnub...
            + PLo  + PCo   + Pcc + Pmisc % [W] Total loss 
Pin = Po+Ploss_total                     % [W] Input power
eta = Po/Pin                             % [dimensionless] Efficiency

% Plot data
t     = linspace(0,Tsw,100);
figure(2); plot(t,I2,"linewidth",2,'b--'); grid on; hold on;
hold on; plot(t,I3,"linewidth",2,'r--'); plot(t,Ip,"linewidth",2,'b');
xlabel('Time [s]'); ylabel('Current [A]')
legend ('Main diode current','FW diode current','Primary current','location','southeast')
title('Current waveforms'); ylim([-4 10]); 